SystemOrganization addCategory: #'SomethingIt-JavaScript-Socket'!

SIExternalInterpreter subclass: #SIJavaScriptInterpreter
	instanceVariableNames: 'socket inputSemaphore'
	classVariableNames: 'HttpServer Undefined'
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIJavaScriptInterpreter class methodsFor: 'class initialization' stamp: 'tomooda 6/18/2013 14:29'!
defaultPage
	"The initial HTML5 page as a JavaScript engine"

	^'<html>
<p id="status">WebSocket: Not connected</p> 
<ul id="log"></ul> 

<script>

var conn = {};
var log = document.getElementById("log");
var state = document.getElementById("status"); 
 
if (window.WebSocket === undefined) {
  state.innerHTML = "(WebSockets not supported)";
} else {
    conn = new WebSocket("ws://localhost:8080/broadcast");
    conn.onopen = function () {
      state.innerHTML = "WebSocket: Connected";
    };
 
    conn.onmessage = function (event) {
	var input = event.data;
	var output;
	try {
		output = JSON.stringify(eval(input));
	} catch (e) {}
	conn.send(output);
      log.innerHTML = "<li>" + input + "<br>" + output + "</li>" + log.innerHTML;
    };
    
    conn.onclose = function (event) {
      state.innerHTML = "WebSocket: Closed";
    };
};
 
</script> 

</body>
</html>'! !

!SIJavaScriptInterpreter class methodsFor: 'class initialization' stamp: 'tomooda 6/18/2013 13:09'!
initialize
	super initialize.
	Undefined ifNil: [ Undefined := Object new]! !

!SIJavaScriptInterpreter class methodsFor: 'class initialization' stamp: 'tomooda 6/18/2013 13:24'!
initializeHttpServer
	"SIJavaScriptInterpreter initializeHttpServer"
	
	HttpServer ifNotNil: [HttpServer destroy].
	HttpServer := WebServer new.
	HttpServer listenOn: 8080.
	HttpServer addService: '/' action:[:req|
		"Serves the main html as a JavaScript engine"
		req send200Response: self defaultPage contentType: 'text/html'.
	].
	HttpServer addService: '/broadcast' action:[:req|
		| socket interpreter |
		"Implements the WebSocket broadcaster"
		socket := req asWebSocket.
		socket timeout: 0.
		interpreter := self on: socket.
		interpreter inspect.
		socket run].! !

!SIJavaScriptInterpreter class methodsFor: 'instance creation' stamp: 'tomooda 6/18/2013 09:24'!
on: aWebSocket
	^ self new setWebSocket: aWebSocket; yourself! !

!SIJavaScriptInterpreter class methodsFor: 'constants' stamp: 'tomooda 6/18/2013 13:09'!
undefined
	^ Undefined! !

!SIJavaScriptInterpreter methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 09:36'!
basicClose
	super basicClose.
	socket
		ifNotNil: [:s | 
			socket := nil.
			s close]! !

!SIJavaScriptInterpreter methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:22'!
expression: aString ifError: errorReceivingBlock 
	^ aString! !

!SIJavaScriptInterpreter methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 10:04'!
initialize
	super initialize.
	inputSemaphore := Semaphore new! !

!SIJavaScriptInterpreter methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:04'!
readUpToPrompt
	inputSemaphore wait.
	^ reader! !

!SIJavaScriptInterpreter methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 10:06'!
readUpToPrompt: anIntegerOrNil
	^ self readUpToPrompt! !

!SIJavaScriptInterpreter methodsFor: 'communicating' stamp: 'tomooda 6/18/2013 09:41'!
send: aString
	socket send: aString! !

!SIJavaScriptInterpreter methodsFor: 'private' stamp: 'tomooda 6/18/2013 10:05'!
setWebSocket: aWebSocket
	socket := aWebSocket.
	socket onClose: [self close].
	socket onMessage: [:data | reader := data. inputSemaphore signal ]! !

!SIJavaScriptInterpreter methodsFor: 'private' stamp: 'tomooda 6/18/2013 10:07'!
startupProcess
	"do nothing"! !

!SIJavaScriptInterpreter methodsFor: 'constant access' stamp: 'tomooda 6/18/2013 13:09'!
undefined
	^ self class undefined! !

Object subclass: #SIJavaScriptValueParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIJavaScriptValueParser methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 15:08'!
initialize
	| digit odigit int octal hex float escChar string bool null undefined array object |
	super initialize.
	digit := #digit asParser.
	odigit := PPPredicateObjectParser chars: '01234567' message: 'octal expected'.
	int := ($+ asParser / $- asParser) optional, digit plus ==> [:pair |
		| abs |
		abs := pair second inject: 0 into: [ :i :c | i * 10 + c digitValue].		
		pair first = $- ifTrue: [abs negated] ifFalse: [abs]].
	octal := ($+ asParser / $- asParser) optional, $0 asParser, odigit plus ==> [:triple |
		| abs |
		abs := triple third inject: 0 into: [ :i :c | i * 8 + c digitValue].		
		triple first = $- ifTrue: [abs negated] ifFalse: [abs]].
	hex := ($+ asParser / $- asParser) optional, '0x' asParser, #hex asParser plus ==> [:triple |
		| abs |
		abs := triple second inject: 0 into: [ :i :c | i * 16 + c digitValue].		
		triple first = $- ifTrue: [abs negated] ifFalse: [abs]].
	float := int, $. asParser, digit plus, (($e asParser / $E asParser), int) optional ==> [:quadraple |
		| order num |
		order := 1.0.
		num := quadraple third inject: quadraple first asFloat into: [ :n :c | c digitValue * (order := order * 0.1) + n].
		quadraple fourth ifNotNilDo: [ :pair | num := num * (10.0 raisedTo: pair second)].
		num].
	escChar := $b asParser ==> [:s | Character backspace ] / ($f asParser ==> [:s | Character newPage]) / ($n asParser ==> [ :s | Character cr] ) / ($r asParser ==> [:s | Character lf]) / ($t asParser ==> [:s | Character tab]) / ($v asParser ==> [:s | Character value: 11]) / (odigit, odigit, odigit ==> [:triple | Character value: (triple first digitValue * 64) + (triple second digitValue * 8) + triple third digitValue]) / ($x asParser, #hex asParser, #hex asParser ==> [:triple | Character value: triple second digitValue * 16 + triple third digitValue]) / ($u asParser, #hex asParser, #hex asParser, #hex asParser, #hex asParser ==> [ :pentaple | Character value: (pentaple second digitValue * 4096) + (pentaple third digitValue * 256) + (pentaple fourth digitValue * 16) + (pentaple fifth digitValue)]) / (#any asParser ==> [:c | c]).
	bool := 'true' asParser ==> [ :s | true] / ('false' asParser ==> [:s | false]).
	null := 'null' asParser ==> [ :s | nil].
	undefined := 'undefined' asParser ==> [:s | SIJavaScriptInterpreter undefined ].
	string := ($" asParser, (($" asParser not, (($\ asParser, escChar ==> [ :pair | pair second]) / #any asParser)) star ==> [ :chars | String withAll: (chars collect: [ :pair | pair last])]), $" asParser) ==> [ :triple | triple second] / (($' asParser, (($' asParser not, (($\ asParser, escChar ==> [ :pair | pair second]) / #any asParser)) star ==> [ :chars | String withAll: (chars collect: [ :pair | pair last])]), $' asParser) ==> [ :triple | triple second]).
	array := PPUnresolvedParser new.
	object := PPUnresolvedParser new.
	parser := PPUnresolvedParser new.
	array def: $[ asParser, parser, ($, asParser, parser ==> [ :pair | pair second]) star, $] asParser ==> [:quad | | tail a | tail := quad third. a := Array new: tail size + 1. a at: 1 put: quad second. 1 to: tail size do: [ :i | a at: i + 1 put: (tail at: i)]. a].
	object def: ${ asParser, (string trim, $: asParser, parser, ($, asParser, string trim, $: asParser, parser) star) optional, $} asParser ==> [:triple | triple second ifNotNil: [:quad | quad fourth inject: (Dictionary new at: quad first put: quad third; yourself) into: [ :d :q | d at: q second put: q fourth; yourself] ] ifNil: [Dictionary new]].
	parser def: (string / bool / null / undefined / array / object / hex / octal / float / int) trim.! !

!SIJavaScriptValueParser methodsFor: 'parsing' stamp: 'tomooda 6/18/2013 11:48'!
parseStream: aStream ifError: errorBlock
	^ parser parse: aStream onError: [errorBlock value: 'Invalid value: ', aStream contents]! !

Object subclass: #SIWebSocketInputStream
	instanceVariableNames: 'buffer semaphore lock empty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SomethingIt-JavaScript-Socket'!

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 10:00'!
basicNext
	^ lock
		critical: [buffer isEmpty
				ifTrue: [empty]
				ifFalse: [| first |
					first := buffer first.
					buffer copyFrom: 2 to: buffer size.
					first]]! !

!SIWebSocketInputStream methodsFor: 'initialize-release' stamp: 'tomooda 6/18/2013 09:59'!
initialize
	super initialize.
	semaphore := Semaphore new.
	lock := Semaphore forMutualExclusion.
	buffer := Array new.
	empty := Object new! !

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 10:02'!
next
	| next |
	[next := self basicNext.
	next = empty]
		whileTrue: [semaphore wait].
	^ next! !

!SIWebSocketInputStream methodsFor: 'accessing' stamp: 'tomooda 6/18/2013 09:53'!
nextPut: aString 
	lock
		critical: [buffer := buffer copyWith: aString.
			buffer size = 1
				ifTrue: [semaphore signal]]! !
SIJavaScriptInterpreter initialize!
